﻿using ProjectSeaplane.Drawnings;
using ProjectSeaplane.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectSeaplane;

public partial class FormPlaneConfig : Form
{
    private DrawingBasicSeaplane _plane;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>

    private event Action<DrawingBasicSeaplane>? PlaneDelegate;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormPlaneConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;
        // TODO buttonCancel.Click привязать анонимный метод через lambda закрытием формы
        buttonCancel.Click += (sender, e) => Close();
    }
    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Привязка внешнего метода к событию
    /// </summary>
    /// <param name="carDelegate"></param>
    public void AddEvent(Action<DrawingBasicSeaplane> planeDelegate)
    {
        PlaneDelegate += planeDelegate;
    }
    /// <summary>
    /// Отрисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _plane?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _plane?.SetPosition(15, 15);
        _plane?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }
    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }
    /// <summary>
    /// Действия при приеме информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _plane = new DrawingBasicSeaplane((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _plane = new DrawingSeaplane((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value,
                Color.White,
                Color.Black, checkBoxLandingGear.Checked,
                checkBoxRadar.Checked);
                break;
        }
        labelBodyColor.BackColor = Color.Empty;
        labelAdditionalColor.BackColor = Color.Empty;
        DrawObject();
    }
    /// <summary>
    /// Передаем информацию при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        // TODO отправка цвета в Drag&Drop
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }
    // TODO Реализовать логику смены цветов: основного и дополнительного (для продвинутого объекта)

    /// <summary>
    /// Проверка получаемой информации по основному цвету
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }
    /// <summary>
    /// Действия при приеме основного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_plane != null)
        {
            _plane.EntityBasicSeaplane.setBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }
    /// <summary>
    /// проверка получаемой информации по доп. цвету
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_plane is DrawingSeaplane)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }

    /// <summary>
    /// Действия при приеме доп. цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_plane.EntityBasicSeaplane is EntitySeaplane _seaplane)
        {
            _seaplane.setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();

    }
    /// <summary>
    /// Передача объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_plane != null)
        {
            PlaneDelegate?.Invoke(_plane);
            Close();
        }
    }


}
