﻿using ProjectSeaplane.Drawnings;
using ProjectSeaplane.MovementStrategy;

namespace ProjectSeaplane
/// <summary>
/// Форма работы с объектом "Гидросамолет"
/// </summary>
{
    public partial class FormSeaplane : Form
    {
        /// <summary>
        /// Конструктор формы
        /// </summary>
        public FormSeaplane()
        {
            InitializeComponent();
            _strategy = null;
        }
        /// <summary>
        /// Поле-объект для прорисовки объекта
        /// </summary>
        private DrawingBasicSeaplane? _drawingSeaplane;
        /// <summary>
        /// Стратегия перемещения
        /// </summary>
        private AbstractStrategy? _strategy;
        /// <summary>
        /// Метод прорисовки самолета
        /// </summary>
        public DrawingBasicSeaplane SetSeaplane
        {
            set
            {
                _drawingSeaplane = value;
                _drawingSeaplane.SetPictureSize(pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
                comboBoxStrategy.Enabled = true;
                _strategy = null;
                Draw();
            }
        }
        private void Draw()
        {
            if (_drawingSeaplane == null)
            {
                return;
            }

            Bitmap bmp = new(pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingSeaplane.DrawTransport(gr);
            pictureBoxSeaplane.Image = bmp;
        }
        
        /// <summary>
        /// Перемещение объекта по форме (нажатие кнопок навигации)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawingSeaplane == null)
            {
                return;
            }

            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result = _drawingSeaplane.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    result = _drawingSeaplane.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    result = _drawingSeaplane.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    result = _drawingSeaplane.MoveTransport(DirectionType.Right);
                    break;
            }

            if (result)
            {
                Draw();
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawingSeaplane == null)
            {
                return;
            }

            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new MoveableSeaplane(_drawingSeaplane), pictureBoxSeaplane.Width, pictureBoxSeaplane.Height);
            }

            if (_strategy == null)
            {
                return;
            }

            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }

        }
    }
}
